from os import getenv

from .modules.process_manager.manager import Manager
from dotenv import load_dotenv, find_dotenv

load_dotenv(find_dotenv())

class Newgrok:

    def __init__(self) -> None:
        port = getenv('app_port')
        chat_id = getenv('chat_id')
        bot_token = getenv('bot_token')
        protocol = getenv('protocol')
        self.manager = Manager(port, protocol, chat_id, bot_token)

    def run(self):
        self.manager.run()


import os
import telebot
from dotenv import load_dotenv, find_dotenv

load_dotenv(find_dotenv())

bot = telebot.TeleBot(os.getenv('bot_token'))

@bot.message_handler(commands=['start'])
def greet(message):
	bot.reply_to(message, f'Chat Id = {message.chat.id}')
	show_chat_id(message)

@bot.message_handler(func=lambda message: True)
def response(message):
	bot.reply_to(message, f'Chat Id = {message.chat.id}')
	show_chat_id(message)

def show_chat_id(message):
	print(f'Chat Id = {message.chat.id}')
	os._exit(0)

def get_chat_id():
	print('Send a message to your bot.')
	bot.infinity_polling()



import telebot

class BotTelegram:

    def __init__(self, chat_id, bot_id):
        self.chat_id = chat_id
        self.bot_id = bot_id

    def send_new_url(self, url: str) -> bool:
        try:
            bot = telebot.TeleBot(self.bot_id)
        except:
            return False

        bot.send_message(self.chat_id, f'🦆 New url - {url}')
        return True
    


import shutil
import atexit
import time
import json
import os
import subprocess
import datetime

import requests
from requests.exceptions import ConnectionError

from ..telegram.bot import BotTelegram

class Manager:
    ngrok_api = 'http://localhost:4040/api/tunnels'
    process = None

    def __init__(self, port: int, protocol: str, chat_id: str, bot_token: str) -> None:
        if not shutil.which('ngrok'):
            print('is ngrok installed?')
            os._exit(0)
        self.port = port
        self.protocol = protocol
        self.chat_id = chat_id
        self.bot_token = bot_token

    def run(self):
        end_time = self.__get_restart_hour()
        while True:
            if self.process == None or end_time < datetime.datetime.now():
                self.__up_process()

    def __get_restart_hour(self) -> datetime.datetime:
        ngrok_start = datetime.datetime.now()
        ngrok_end = ngrok_start + datetime.timedelta(hours=1.5)
        return ngrok_end

    def __up_process(self):
        self.__stop_ngrok()
        self.process = subprocess.Popen(
            ['ngrok', 'http', '--inspect=false', str(self.port)],
            stdin=subprocess.PIPE, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT
        )
        self.__send_new_url()
        self.__register_quit_ngrok_process(self.process.terminate)

    def __stop_ngrok(self) -> None:
        if self.process != None:
            self.process.terminate()

    def __register_quit_ngrok_process(self, func: callable) -> None:
        atexit._clear()
        atexit.register(func)

    def __get_public_url(self) -> str:
        attempts = 10
        for attempt in range(attempts):
            try:
                response = requests.get(self.ngrok_api)
                break
            except ConnectionError:
                time.sleep(2)
            if attempt >= attempts -1:
                print('is ngrok running?')
                os._exit(0)
        data = response.text
        tunnel_url = json.loads(data)['tunnels'][0]['public_url']
        return tunnel_url

    def __send_new_url(self):
        url = self.__get_public_url()
        bot = BotTelegram(self.chat_id, self.bot_token)
        bot.send_new_url(url)


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['newgrok',
 'newgrok.cli',
 'newgrok.modules.process_manager',
 'newgrok.modules.telegram']

package_data = \
{'': ['*']}

install_requires = \
['pytelegrambotapi>=4.8.0,<5.0.0',
 'python-dotenv>=0.21.0,<0.22.0',
 'requests>=2.28.1,<3.0.0']

entry_points = \
{'console_scripts': ['chat_id = newgrok.cli.chat:get_chat_id']}

setup_kwargs = {
    'name': 'newgrok',
    'version': '0.4.0',
    'description': 'Keep ngrok free tier alive',
    'long_description': '<p align="center">\n  <a href="https://user-images.githubusercontent.com/101568457/208002599-be7000ca-e05a-4706-99b2-49bc5a35e367.png"><img src="https://user-images.githubusercontent.com/101568457/208002599-be7000ca-e05a-4706-99b2-49bc5a35e367.png" alt="FastAPI"></a>\n</p>\n<p align="center">\n    <em>Keep your ngrok free tier alive, forever</em>\n</p>\n\n---\n<h1>\n! Use this tool for personal use only, for prototyping projects and related things. For commercial or production use consider subscribing to ngrok.\n</h1>\n\n## Prerequisites\n* Ngrok already configured with your key\n* A bot on telegram\n---\n## installation\n\n* With pip\n    ```\n    pip3 newgrok\n    ```\n\n* With poetry:\n\n    ```\n    poetry add newgrok\n    ```\n---\n## Variables\n| Variable  |                                                 |\n|-----------|-------------------------------------------------|\n| app_port  | Port of the application running on your machine |\n| protocol  | Network protocol you want to use                |\n| bot_token | Your bot token in telegram                      |\n| chat_id   | The id of the chat between the bot and you      |\n---\n## Setup\n\n* First you need to set the environment variables, chat_id for later, edit the ```.env``` file\n    ```python\n    app_port=8000\n    protocol=http\n    bot_token=******************************\n    chat_id=\n    ```\n* To get the chat_id, run:\n    ```\n    chat_id\n    ```\n    Now send a message to the bot, and it will reply with the chat_id. Complete the ```.env``` file\n    ```python\n    app_port=8000\n    protocol=http\n    bot_token=******************************\n    chat_id=********\n    ```\n* Now with these 3 lines of code, newgrok is already running, enjoy\n    ```python\n    from newgrok import Newgrok\n\n    ngrok = Newgrok()\n    ngrok.run()\n    ```\n',
    'author': 'Adrien',
    'author_email': 'adrienv.sui@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
